/**
 * @file
 * Implements our progress bar, replacing Turbo's implementation.
 *
 * @see https://www.drupal.org/project/refreshless/issues/3488464
 *   Issue for adapting this out of Turbo.
 */
((html, ProgressBar, ProgressBarDelay, $, once) => { $(once(
  'refreshless-turbo-progress-bar', html,
)).each(() => {

  'use strict';

  const progressBarDelay = new ProgressBarDelay(
    Turbo.session.progressBarDelay, new ProgressBar(),
  );

  html.refreshlessDocumentProgressBar = progressBarDelay;

  // Note that this intentionally listens to the fetch request event rather than
  // turbo:click so that the progress bar also kicks in during back/forward
  // history navigation.
  $(html).on('turbo:before-fetch-request', (event) => {

    if (
      // Ignore prefetch requests.
      'X-Sec-Purpose' in event.detail.fetchOptions.headers &&
      event.detail.fetchOptions.headers['X-Sec-Purpose'] === 'prefetch' ||
      // We want to specifically ignore forms as those are handled separately
      // via the turbo:submit-start and turbo:submit-end event handlers.
      $(event.target).is('form') === true
    ) {
      return;
    }

    progressBarDelay.showVisitAfterDelay();

    const hide = () => {

      progressBarDelay.hideVisit();

      $(html).off([
        'refreshless:before-render',
        'refreshless:reload',
        'turbo:fetch-request-error',
      ].join(' '), hide);

    };

    $(html).one([
      // We're using refreshless:before-render rather than refreshless:load
      // because the progress bar is noticeably less smoothly transitioned out
      // when loading some complex/heavy pages on mobile in some browsers,
      // specifically Firefox on Android. The refreshless:before-render is
      // early enough that it should give the progress bar enough time to
      // transition out before the new <body> contents are rendered.
      'refreshless:before-render',
      // If a reload occurs as a result of this navigation, hide the progress
      // bar. Since we're using the before-render event, that would never get
      // triggered during a reload since Turbo isn't about to render anything.
      'refreshless:reload',
      'turbo:fetch-request-error',
    ].join(' '), hide);

  });

  $(html).on('turbo:submit-start', (event) => {

    progressBarDelay.showFormAfterDelay();

    const hide = () => {

      progressBarDelay.hideForm();

      $(html).off([
        'turbo:fetch-request-error',
        'turbo:submit-end',
      ].join(' '), hide);

    };

    $(html).one([
      'turbo:submit-end',
      'turbo:fetch-request-error',
    ].join(' '), hide);

  });

}); })(
  document.documentElement,
  Drupal.RefreshLess.classes.ProgressBar,
  Drupal.RefreshLess.classes.ProgressBarDelay,
  jQuery,
  once,
);
