((Drupal, $) => {

  'use strict';

  if (!('RefreshLess' in Drupal)) {
    Drupal.RefreshLess = {};
  }

  if (!('classes' in Drupal.RefreshLess)) {
    Drupal.RefreshLess.classes = {};
  }

  /**
   * RefreshLess Turbo script manager class.
   */
  Drupal.RefreshLess.classes.TurboScriptManager = class {

    /**
     * The context element to attach to; usually the <html> element.
     *
     * @type {HTMLElement}
     */
    #context;

    constructor(context) {

      this.#context = context;

      this.#bindEventHandlers();

    }

    /**
     * Bind all of our event handlers.
     */
    #bindEventHandlers() {

      $(this.#context).on({
        'turbo:before-scripts-merge': (event) => {
          this.#beforeMergeHandler(event);
        },
        'turbo:scripts-merged': (event) => {
          this.#mergedHandler(event);
        },
      });

    }

    /**
     * 'turbo:before-scripts-merge' event handler.
     *
     * @param {jQuery.Event} event
     */
    #beforeMergeHandler(event) {

      const beforeMergeEvent = new CustomEvent(
        'refreshless:before-scripts-merge', {
          detail: event.detail,
        },
      );

      this.#context.dispatchEvent(beforeMergeEvent);

    }

    /**
     * 'turbo:scripts-merged' event handler.
     *
     * @param {jQuery.Event} event
     */
    #mergedHandler(event) {

      const mergeEvent = new CustomEvent(
        'refreshless:scripts-merged', {
          detail: event.detail,
        },
      );

      this.#context.dispatchEvent(mergeEvent);

    }

  }

})(Drupal, jQuery);
